//
// Created by yelv1 on 24-12-31.
//

#include "ultrasonic_analog.h"

volatile bool ultrasonicMeasurementDone = false;
extern uint32_t g_capture_value;

/*!
    \brief      configure ultrasonic gpio & timer13 ch0 pwm output
    \param[in]  none
    \param[out] none
    \retval     none
*/
void ultrasonic_gpio_config(void) {
    rcu_periph_clock_enable(US_TX_GPIO_RCU);

    gpio_mode_set(US_TX_GPIO_PORT, GPIO_MODE_AF, GPIO_PUPD_NONE, US_TX_PIN);
    gpio_output_options_set(US_TX_GPIO_PORT, GPIO_OTYPE_PP, GPIO_OSPEED_50MHZ, US_TX_PIN);
    gpio_af_set(US_TX_GPIO_PORT, US_TX_GPIO_AF, US_TX_PIN);

    timer_oc_parameter_struct timer_ocinitpara;
    timer_parameter_struct timer_initpara;

    rcu_periph_clock_enable(US_TX_RCU);
    timer_deinit(US_TX_TIMER);

    timer_struct_para_init(&timer_initpara);
    timer_initpara.prescaler = 0;
    timer_initpara.alignedmode = TIMER_COUNTER_EDGE;
    timer_initpara.counterdirection = TIMER_COUNTER_UP;
    timer_initpara.period = 239;
    timer_initpara.clockdivision = TIMER_CKDIV_DIV1;
    timer_init(US_TX_TIMER, &timer_initpara);

    timer_channel_output_struct_para_init(&timer_ocinitpara);
    timer_ocinitpara.outputstate = TIMER_CCX_ENABLE;
    timer_ocinitpara.outputnstate = TIMER_CCXN_DISABLE;
    timer_ocinitpara.ocpolarity = TIMER_OC_POLARITY_HIGH;
    timer_ocinitpara.ocnpolarity = TIMER_OCN_POLARITY_HIGH;
    timer_ocinitpara.ocidlestate = TIMER_OC_IDLE_STATE_LOW;
    timer_ocinitpara.ocnidlestate = TIMER_OCN_IDLE_STATE_LOW;
    timer_channel_output_config(US_TX_TIMER, US_TX_CH, &timer_ocinitpara);

    timer_channel_output_pulse_value_config(US_TX_TIMER, US_TX_CH, 120);
    timer_channel_output_mode_config(US_TX_TIMER, US_TX_CH, TIMER_OC_MODE_PWM0);
    timer_auto_reload_shadow_enable(US_TX_TIMER);

    timer_interrupt_enable(US_TX_TIMER, TIMER_INT_UP);
}

void ultrasonic_pwm_out_cycles(const uint8_t cycles) {
    ultrasonicMeasurementDone = false;
    uint8_t current_cycle = 0;

    timer_channel_output_pulse_value_config(US_TX_TIMER, US_TX_CH, 120);
    timer_channel_output_mode_config(US_TX_TIMER, US_TX_CH, TIMER_OC_MODE_PWM1);
    timer_enable(US_TX_TIMER);

    timer_enable(US_TX_DELAY_TIMER);

    while (current_cycle < cycles)
    {
        while (!timer_interrupt_flag_get(US_TX_TIMER, TIMER_INT_FLAG_UP));
        timer_interrupt_flag_clear(US_TX_TIMER, TIMER_INT_FLAG_UP);
        current_cycle++;
    }
    // delay_nop();
    timer_disable(US_TX_TIMER);
}

/*!
    \brief      configure ultrasonic transmit debounce delay(timer15) 72MHz/72 = 1MHz 1us.
    \param[in]  micro_second: delay time in micro second
    \param[out] none
    \retval     none
*/
void ultrasonic_transmit_debounce_delay(const uint16_t micro_second) {
    rcu_periph_clock_enable(US_TX_DELAY_RCU);
    timer_deinit(US_TX_DELAY_TIMER);

    timer_parameter_struct timer_initpara;
    timer_struct_para_init(&timer_initpara);
    timer_initpara.prescaler = 71;
    timer_initpara.alignedmode = TIMER_COUNTER_EDGE;
    timer_initpara.counterdirection = TIMER_COUNTER_UP;
    timer_initpara.period = micro_second - 1;
    timer_initpara.clockdivision = TIMER_CKDIV_DIV1;
    timer_initpara.repetitioncounter = 0;
    timer_init(US_TX_DELAY_TIMER, &timer_initpara);

    timer_auto_reload_shadow_enable(US_TX_DELAY_TIMER);
    timer_interrupt_enable(US_TX_DELAY_TIMER, TIMER_INT_UP);
    nvic_irq_enable(TIMER15_IRQn, 1U);
}

void ultrasonic_receive_exti_config(void) {
    rcu_periph_clock_enable(US_RX_GPIO_RCU);
    rcu_periph_clock_enable(US_RX_EXTI_RCU);

    gpio_mode_set(US_RX_GPIO_PORT, GPIO_MODE_INPUT, GPIO_PUPD_NONE, US_RX_GPIO_PIN);
    nvic_irq_enable(US_RX_EXTI_IRQ, 0U);
    syscfg_exti_line_config(EXTI_SOURCE_GPIOA, US_RX_EXTI_LINE);

    exti_init(US_RX_GPIO_EXTI, EXTI_INTERRUPT, EXTI_TRIG_FALLING);
    exti_flag_clear(US_RX_GPIO_EXTI);

    // exti_interrupt_enable(EXTI_0);
}

void ultrasonic_echo_timer_config(void) {
    rcu_periph_clock_enable(US_ECHO_RCU);
    timer_deinit(US_ECHO_TIMER);

    timer_parameter_struct timer_initpara;
    timer_struct_para_init(&timer_initpara);
    timer_initpara.prescaler = 71;
    timer_initpara.alignedmode = TIMER_COUNTER_EDGE;
    timer_initpara.counterdirection = TIMER_COUNTER_UP;
    timer_initpara.period = 59999;
    timer_initpara.clockdivision = TIMER_CKDIV_DIV1;
    timer_initpara.repetitioncounter = 0;
    timer_init(US_ECHO_TIMER, &timer_initpara);

    timer_ic_parameter_struct timer_icinitpara;
    timer_channel_input_struct_para_init(&timer_icinitpara);
    timer_icinitpara.icpolarity = TIMER_IC_POLARITY_BOTH_EDGE;
    timer_icinitpara.icselection = TIMER_IC_SELECTION_INDIRECTTI;
    timer_icinitpara.icprescaler = TIMER_IC_PSC_DIV1;
    timer_icinitpara.icfilter = 0x03;
    timer_input_capture_config(US_ECHO_TIMER, US_ECHO_CH, &timer_icinitpara);
}

void ultrasonic_config(void) {
    ultrasonic_gpio_config();
    ultrasonic_transmit_debounce_delay(TIME_CORRECTION_US);
    ultrasonic_receive_exti_config();
    ultrasonic_echo_timer_config();
}

uint16_t ultrasonic_calc_distance(void) {
    while (!ultrasonicMeasurementDone);
    // uint32_t us_value = timer_channel_capture_value_register_read(US_ECHO_TIMER, US_ECHO_CH);
    uint16_t distance = (TIME_CORRECTION_US + g_capture_value) * 17;
    /*
     *  (TIME_CORRECTION_US + us_value) * 340 m/s
     *  -----------------------------------------
     *                  1000 000
     *  ----------------------------------------------
     *                          2
     */
    return distance;
}